﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.Threading;
using System.Runtime.InteropServices;

namespace BatteryCheckInfo
{
    public partial class FormMain : Form
    {
        [DllImport(@"WMIO2.dll")]
        public static extern bool GetBattery1SpecificInfo(uint uiCommand, out uint uiValue);

        [DllImport(@"WMIO2.dll")]
        public static extern bool GetBattery2SpecificInfo(uint uiCommand, out uint uiValue);

        [DllImport(@"WMIO2.dll")]
        public static extern bool GetBackupBatteryStatus(out uint uiValue);

        public static string c_szVersion = Assembly.GetExecutingAssembly().GetName().Name.ToString() + " " + "V" + Assembly.GetExecutingAssembly().GetName().Version.ToString();
        private static uint[] Bat1Info = new uint[0x20];

        private static uint[] SpecBatIndex = { 0x03, 0x08, 0x09, 0x0a, 0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1b };
        private static uint iSpecBatIndex = 0;

        public FormMain()
        {
            InitializeComponent();

            for (int i = 0; i < 0x20; i++)
            {
                Bat1Info[i] = 0;
            }
        }

        private void FormMain_Load(object sender, EventArgs e)
        {

            this.Text = c_szVersion;

            System.Diagnostics.Process[] myProcesses = System.Diagnostics.Process.GetProcessesByName("BatteryCheckInfo");
            if (myProcesses.Length > 1)
            {
                //MessageBox.Show("Already running.");
                //Environment.Exit(1);
            }

            uint iTemp=0;
            if (GetBattery1SpecificInfo(0xF0, out iTemp))
            {

            }

            if (iTemp != 0xaaaa)
            {
                DisableUI();
                this.Text = c_szVersion + "(Not Support)";
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            button1.Enabled = false;
            CleanDisplayData();
            timer1.Start();
        }

        private void update()
        {
            uint iTemp = 0;
            float Temperature = 0;

            for (iSpecBatIndex = 0; iSpecBatIndex < 18; iSpecBatIndex++)
            {
                iTemp = 0;

                if (radioButton1.Checked)
                {
                    if (GetBattery1SpecificInfo(SpecBatIndex[iSpecBatIndex], out iTemp))
                        Bat1Info[SpecBatIndex[iSpecBatIndex]] = iTemp;
                }
                else
                {
                    if (GetBattery2SpecificInfo(SpecBatIndex[iSpecBatIndex], out iTemp))
                        Bat1Info[SpecBatIndex[iSpecBatIndex]] = iTemp;            
                }

                //BatteryMode
                textBox1.Text = String.Format("0x{0:X}", Bat1Info[0x03]);

                //Temperature
                if (Bat1Info[0x08] == 0x8000)
                {
                    textBox2.Text = "N/A";
                }
                else
                {
                    Temperature = (float)((float)Bat1Info[0x08] * 0.1 - (float)273);
                    textBox2.Text = Temperature.ToString();
                }

                //Voltage
                textBox3.Text = Bat1Info[0x09].ToString();

                //Current
                int tmp = 0;
                if ((Bat1Info[0x0a] & 0x8000) == 0x8000)
                {
                    tmp = (int)(0xFFFF - Bat1Info[0x0a]);
                    tmp = -tmp;
                }
                else
                {
                    tmp = (int)Bat1Info[0x0a];
                }
                textBox4.Text = tmp.ToString();

                //
                textBox5.Text = Bat1Info[0x0D].ToString();
                textBox6.Text = Bat1Info[0x0E].ToString();
                textBox7.Text = Bat1Info[0x0F].ToString();
                textBox8.Text = Bat1Info[0x10].ToString();
                textBox9.Text = Bat1Info[0x11].ToString();
                textBox10.Text = Bat1Info[0x12].ToString();
                textBox11.Text = Bat1Info[0x13].ToString();
                textBox12.Text = Bat1Info[0x14].ToString();
                textBox13.Text = Bat1Info[0x15].ToString();

                //BatteryStatus
                textBox14.Text = String.Format("0x{0:X}", Bat1Info[0x16]);

                //
                textBox15.Text = Bat1Info[0x17].ToString();
                textBox16.Text = Bat1Info[0x18].ToString();
                textBox17.Text = Bat1Info[0x19].ToString();

                //ManufactureDate

                // Day : bit4 - bit0
                // Month : bit8 - bit5
                // Year : bit15 - bit9 (by 1980)

                int day = 0;
                int month = 0;
                int year = 1980;

                day = (int)(Bat1Info[0x1b] & 0x1F);
                month = (int)((Bat1Info[0x1b] >> 5) & 0x0F);
                year += (int)((Bat1Info[0x1b] >> 9) & 0x7F);

                textBox18.Text = year.ToString() + "/" + month.ToString() + "/" + day.ToString();

            }
        }

        private void CleanDisplayData()
        {
            textBox1.Text = "";
            textBox2.Text = "";
            textBox3.Text = "";
            textBox4.Text = "";
            textBox5.Text = "";
            textBox6.Text = "";
            textBox7.Text = "";
            textBox8.Text = "";
            textBox9.Text = "";
            textBox10.Text = "";
            textBox11.Text = "";
            textBox12.Text = "";
            textBox13.Text = "";
            textBox14.Text = "";
            textBox15.Text = "";
            textBox16.Text = "";
            textBox17.Text = "";
            textBox18.Text = "";
        }

        private void DisableUI()
        {
            groupBox1.Enabled = false;

            radioButton1.Enabled = false;
            radioButton2.Enabled = false;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            CleanDisplayData();
            groupBox1.Enabled = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            timer1.Stop();
            update();
            button1.Enabled = true;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            CleanDisplayData();
            groupBox1.Enabled = true;
        }

    }
}
