

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Reflection;
using System.Diagnostics;


namespace ECcomm
{
    public partial class FormHotTabHook : Form
    {

        #region User32 DLL Declare For All

        [DllImport("user32.dll")]
        public static extern void keybd_event(byte bVk, byte bScan, uint dwFlags, uint dwExtraInfo);

        [DllImport("user32.dll")]
        public static extern Int32 PostMessage(Int32 hWnd, Int32 wMsg, Int32 wParam, Int32 lParam);

        [DllImport("user32.dll")]
        public static extern Int32 FindWindow(String lpClassName, String lpWindowName);

        [DllImport("user32.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);


        #endregion

        #region Windows structure definitions

        [StructLayout(LayoutKind.Sequential)]
        private class POINT
        {
            public int x;
            public int y;
        }


        [StructLayout(LayoutKind.Sequential)]
        private class KeyboardHookStruct
        {
            public int vkCode;
            public int scanCode;
            public int flags;
            public int time;
            public int dwExtraInfo;
        }
        #endregion

        #region Windows function imports

        [DllImport("kernel32.dll")]
        public static extern IntPtr GetModuleHandle(string name);

        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall, SetLastError = true)]
        private static extern int SetWindowsHookEx(int idHook, HookProc lpfn, IntPtr hMod, int dwThreadId);

        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall, SetLastError = true)]
        private static extern int UnhookWindowsHookEx(int idHook);

        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall)]
        private static extern int CallNextHookEx(int idHook, int nCode, int wParam, IntPtr lParam);

        private delegate int HookProc(int nCode, int wParam, IntPtr lParam);

        [DllImport("user32")]
        private static extern int ToAscii(int uVirtKey, int uScanCode, byte[] lpbKeyState, byte[] lpwTransKey, int fuState);

        [DllImport("user32")]
        private static extern int GetKeyboardState(byte[] pbKeyState);

        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall)]
        private static extern short GetKeyState(int vKey);

        #endregion

        #region Windows constants

        private const int WH_MOUSE_LL = 14;
        private const int WH_KEYBOARD_LL = 13;
        private const int WH_MOUSE = 7;
        private const int WH_KEYBOARD = 2;

        private const int WM_MOUSEMOVE = 0x200;
        private const int WM_LBUTTONDOWN = 0x201;
        private const int WM_RBUTTONDOWN = 0x204;
        private const int WM_MBUTTONDOWN = 0x207;
        private const int WM_LBUTTONUP = 0x202;
        private const int WM_RBUTTONUP = 0x205;
        private const int WM_MBUTTONUP = 0x208;
        private const int WM_LBUTTONDBLCLK = 0x203;
        private const int WM_RBUTTONDBLCLK = 0x206;
        private const int WM_MBUTTONDBLCLK = 0x209;
        private const int WM_MOUSEWHEEL = 0x020A;

        private const int WM_KEYDOWN = 0x100;
        private const int WM_KEYUP = 0x101;
        private const int WM_SYSKEYDOWN = 0x104;
        private const int WM_SYSKEYUP = 0x105;

        private const byte VK_SHIFT = 0x10;
        private const byte VK_CAPITAL = 0x14;
        private const byte VK_NUMLOCK = 0x90;

        #endregion

        #region Hook Original Event Handler

        public event KeyEventHandler KeyDownForHook;
        public event KeyPressEventHandler KeyPressForHook;
        public event KeyEventHandler KeyUpForHook;

        #endregion

        #region Hook Original Attribute Declare

        private int hKeyboardHook = 0;
        private HookProc KeyboardHookProcedure;

        #endregion

        #region HotTab Keyboard Hook Const & Enumeration Declare

        private enum KeyboardInputState
        {
            State_None,
            State_StartWithAlt,
            State_StartWithCtrl,
            State_StartWithAltCtrl,
            State_StartWithCtrlAlt,
            State_StartWithAltCtrlShift,
            State_StartWithCtrlAltShift,
        }

        #endregion

        #region HotTab Attribute Declare

        private Int32 hwndMainForm;
        private KeyboardInputState keyboardInputState;
        private DateTime beforeKeyboardInputStateDateTime;
        private TimeSpan KeyboardInputDifferenceTime;

        #endregion

        public FormHotTabHook()
        {
            InitializeComponent();
            Start();
        }
        public FormHotTabHook(bool InstallKeyboardHook)
        {
            InitializeComponent();
            Start(InstallKeyboardHook);
        }
        ~FormHotTabHook()
        {
            Stop(true, false);
        }

        public void Start()
        {
            this.Start(true);
        }
        public void Start(bool InstallKeyboardHook)
        {
            if (hKeyboardHook == 0 && InstallKeyboardHook)
            {
                KeyboardHookProcedure = new HookProc(KeyboardHookProc);
                hKeyboardHook = SetWindowsHookEx(WH_KEYBOARD_LL, KeyboardHookProcedure, GetModuleHandle(Process.GetCurrentProcess().MainModule.ModuleName), 0);

                if (hKeyboardHook == 0)
                {
                    int errorCode = Marshal.GetLastWin32Error();
                    Stop(true, false);
                    throw new Win32Exception(errorCode);
                }
                else
                {
                    beforeKeyboardInputStateDateTime = DateTime.Now;
                    keyboardInputState = KeyboardInputState.State_None;
                    hwndMainForm = FindWindow(null, "ECcomm");
                    this.KeyUpForHook += new KeyEventHandler(this.KeyboradUpHandler);
                    //this.KeyDownForHook += new KeyEventHandler(this.KeyboradDownHandler);
                }
            }
        }

        public void Stop()
        {
            this.Stop(true, true);
        }

        public void Stop(bool UninstallKeyboardHook, bool ThrowExceptions)
        {
            if (hKeyboardHook != 0 && UninstallKeyboardHook)
            {
                int retKeyboard = UnhookWindowsHookEx(hKeyboardHook);
                hKeyboardHook = 0;
                if (retKeyboard == 0 && ThrowExceptions)
                {
                    int errorCode = Marshal.GetLastWin32Error();
                    throw new Win32Exception(errorCode);
                }
            }
        }

        private int KeyboardHookProc(int nCode, Int32 wParam, IntPtr lParam)
        {
            bool handled = false;
            KeyboardHookStruct MyKeyboardHookStruct = (KeyboardHookStruct)Marshal.PtrToStructure(lParam, typeof(KeyboardHookStruct));
            if ((nCode >= 0) && (KeyDownForHook != null || KeyUpForHook != null || KeyPressForHook != null))
            {
                if (KeyDownForHook != null && (wParam == WM_KEYDOWN || wParam == WM_SYSKEYDOWN))
                {
                    Keys keyData = (Keys)MyKeyboardHookStruct.scanCode;
                    KeyEventArgs e = new KeyEventArgs(keyData);
                    KeyDownForHook(this, e);
                    handled = handled || e.Handled;
                }

                if (KeyPressForHook != null && wParam == WM_KEYDOWN)
                {
                    bool isDownShift = ((GetKeyState(VK_SHIFT) & 0x80) == 0x80 ? true : false);
                    bool isDownCapslock = (GetKeyState(VK_CAPITAL) != 0 ? true : false);

                    byte[] keyState = new byte[256];
                    GetKeyboardState(keyState);
                    byte[] inBuffer = new byte[2];
                    if (ToAscii(MyKeyboardHookStruct.vkCode, MyKeyboardHookStruct.scanCode, keyState, inBuffer, MyKeyboardHookStruct.flags) == 1)
                    {
                        char key = (char)inBuffer[0];
                        if ((isDownCapslock ^ isDownShift) && Char.IsLetter(key)) key = Char.ToUpper(key);
                        KeyPressEventArgs e = new KeyPressEventArgs(key);
                        KeyPressForHook(this, e);
                        handled = handled || e.Handled;
                    }
                }

                if (KeyUpForHook != null && (wParam == WM_KEYUP || wParam == WM_SYSKEYUP))
                {
                    Keys keyData = (Keys)MyKeyboardHookStruct.scanCode;
                    //Trace.WriteLine("vkCode=" + MyKeyboardHookStruct.vkCode.ToString("X2") + ", scanCode=" + MyKeyboardHookStruct.scanCode.ToString("X2"));
                    KeyEventArgs e = new KeyEventArgs(keyData);
                    KeyUpForHook(this, e);
                    handled = handled || e.Handled;
                }
            }

            if (handled)
                return 1;
            else
                return CallNextHookEx(hKeyboardHook, nCode, wParam, lParam);
        }

        private void KeyboradUpHandler(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 541) return; //French Mode, Because the alt key will send the code.

            KeyboardInputDifferenceTime = DateTime.Now - beforeKeyboardInputStateDateTime;
            beforeKeyboardInputStateDateTime = DateTime.Now;
            if (KeyboardInputDifferenceTime.TotalMilliseconds > 200)
            {
                keyboardInputState = KeyboardInputState.State_None;
                beforeKeyboardInputStateDateTime = DateTime.Now;
            }
            else
            {
                beforeKeyboardInputStateDateTime = DateTime.Now;
            }

            switch (keyboardInputState)
            {
                case KeyboardInputState.State_None:
                    switch (e.KeyValue)
                    {
                        case 0x38: //alt
                            keyboardInputState = KeyboardInputState.State_StartWithAlt;
                            break;
                        case 0x1D: //ctrl
                            keyboardInputState = KeyboardInputState.State_StartWithCtrl;
                            break;

                        case 0x73:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08BrightnessAuto, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x74:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08BrightnessDown, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x75:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08BrightnessUp, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x67:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08VolumeUp, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x77:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08VolumeDown, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x70:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08VolumeMute, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x7D:
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_FM08Lamp, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;

                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithAlt:
                    switch (e.KeyValue)
                    {
                        case 29: //ctrl
                            keyboardInputState = KeyboardInputState.State_StartWithAltCtrl;
                            break;
                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithCtrl:
                    switch (e.KeyValue)
                    {
                        case 56: //alt
                            keyboardInputState = KeyboardInputState.State_StartWithCtrlAlt;
                            break;
                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithAltCtrl:
                    switch (e.KeyValue)
                    {
                        case 54: //shift
                            keyboardInputState = KeyboardInputState.State_StartWithAltCtrlShift;
                            break;
                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithCtrlAlt:
                    switch (e.KeyValue)
                    {
                        case 54: //shift
                            keyboardInputState = KeyboardInputState.State_StartWithCtrlAltShift;
                            break;
                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithAltCtrlShift:

                    switch (e.KeyValue)
                    {
                        case 0x0B: //0 (PS2 Set:B)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_Menu, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;

                        case 0x02: //1 (PS2 Set:2)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_VolumeUp, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x03: //2 (PS2 Set:3)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_VolumeDown, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x0A: //9 (PS2 Set:A)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_Left, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x1E: //a (PS2 Set:1E)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_Right, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x12: //e (PS2 Set:12)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_OK, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;

                        case 0x04: //3 (PS2 Set:4)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F1Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x05: //4 (PS2 Set:5)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F1Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x06: //5 (PS2 Set:6)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F2Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x07: //6 (PS2 Set:7)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F2Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x08: //7 (PS2 Set:8)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F3Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x09: //8 (PS2 Set:9)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F3Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x31: //n (PS2 Set:31)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F4Short, 0, 0);
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_AirDockingIn, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x21: //f (PS2 Set:21)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F4Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x22: //g (PS2 Set:22)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F5Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x23: //h (PS2 Set:23)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F5Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x17: //i (PS2 Set:17)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F6Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x24: //j (PS2 Set:24)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F6Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x25: //k (PS2 Set:25)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F7Short, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x26: //l (PS2 Set:26)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_F7Long, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x20: //d (PS2 Set:20)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_HomeShort, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x32: //m (PS2 Set:32)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_HomeLong, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;

                        case 0x10: //q (PS2 Set:10)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_DockingOut, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                        case 0x19: //p (PS2 Set:19)
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_GroundDockingIn, 0, 0);
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                 

                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
                case KeyboardInputState.State_StartWithCtrlAltShift:
                    switch (e.KeyValue)
                    {
                        case 31: //s; lightsensor triger
                            PostMessage(hwndMainForm, HotTabECEventArgs.ECEvent_LightSensorEvent, 0, 0);
                            break;
                        default:
                            keyboardInputState = KeyboardInputState.State_None;
                            break;
                    }
                    break;
            }
        }


    }
}