﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace PackagePower
{
    public partial class Form1 : Form
    {
        [DllImport(@"WMIO2.dll")]
        public static extern bool GetCPUTemperature(out byte value);

        [DllImport(@"WMIO2.dll")]
        public static extern bool GetCPUPL1(out byte value);

        [DllImport(@"WMIO2.dll")]
        public static extern bool SetCPUPL1(byte value);

        public static string c_szVersion = Assembly.GetExecutingAssembly().GetName().Name.ToString() + " " + "V" + Assembly.GetExecutingAssembly().GetName().Version.ToString();
        
        //Package Power Table for M101B/M9020B/IBCMC/FM10/FM08/ALG10XB/M101BK
        private static string[] PackagePowerLimitLevel1 = { "1.000", "1.500", "2.000", "2.500", "3.000", "3.500", "4.000", "4.500", "5.000", "5.500", "6.000", "6.500", "7.000", "7.500" };
        //Package Power Table for M101H/IH83/M133
        private static string[] PackagePowerLimitLevel2 = { "1.000", "1.500", "2.000", "2.500", "3.000", "3.500", "4.000", "4.500", "5.000", "5.500", "6.000", "6.500", "7.000", "7.500", "8.000", "8.500", "9.000", "9.500", "10.000", "10.500", "11.000", "11.500", "12.000", "12.500", "13.000", "13.500", "14.000", "14.500", "15.000" };
        //Package Power Table for M800BW/Altas8
        private static string[] PackagePowerLimitLevel3 = { "1.024", "1.536", "2.048", "2.560", "3.072", "3.584", "4.096", "4.608", "5.120", "5.632" };
        //Package Power Table for IKCMC
        private static string[] PackagePowerLimitLevel4 = { "1.000", "1.500", "2.000", "2.500", "3.000", "3.500", "4.000", "4.500", "5.000", "5.500", "6.000", "6.500", "7.000" };
    
        private int iCount = 0;
        private int iTemperature = 0;
        private int iTemperatureAverage = 0;

        private int iMaximumIndex = 0;

        /*
         *  iPlatformIndex = 0 , for M101B/M9020B/IBCMC/FM10/FM08/ALG10XB/M101BK
         *  iPlatformIndex = 1 , for M101H/IH83/M133
         *  iPlatformIndex = 2 , for M800BW/Altas8
         *  iPlatformIndex = 3 , for IKCMC
         */
        private int iPlatformIndex = 0;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Text = c_szVersion;
            PlatformComboBox.SelectedIndex = 0;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (iCount < 10)
            {
                byte value;
                if (GetCPUTemperature(out value))
                {
                    iTemperature += value;
                    iCount++;
                }
            }
            else
            {
                iTemperatureAverage = iTemperature / iCount;
                CPUTempLabel.Text = "CPU Temp. : " + iTemperatureAverage.ToString() + " Degree C";
                
                iTemperatureAverage = 0;
                iTemperature = 0;
                iCount = 0;
            }
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            byte value2;
            if (GetCPUPL1(out value2))
            {
                if (value2 > iMaximumIndex - 1)
                    CPUPL1Label.Text = "CPU PL1 : Level" + value2.ToString() + ", " + "Exceeds the maximum limit.";
                else if (iPlatformIndex == 0)
                    CPUPL1Label.Text = "CPU PL1 : Level" + value2.ToString() + ", " + PackagePowerLimitLevel1[value2].ToString() + " Watts";
                else if (iPlatformIndex == 1)
                    CPUPL1Label.Text = "CPU PL1 : Level" + value2.ToString() + ", " + PackagePowerLimitLevel2[value2].ToString() + " Watts";
                else if (iPlatformIndex == 2)
                    CPUPL1Label.Text = "CPU PL1 : Level" + value2.ToString() + ", " + PackagePowerLimitLevel3[value2].ToString() + " Watts";
                else if (iPlatformIndex == 3)
                    CPUPL1Label.Text = "CPU PL1 : Level" + value2.ToString() + ", " + PackagePowerLimitLevel4[value2].ToString() + " Watts";
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (!SetCPUPL1((byte)PL1ComboBox.SelectedIndex))
            {
                MessageBox.Show("fail", "Warning");
            }
        }

        private void PlatformComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            iPlatformIndex = PlatformComboBox.SelectedIndex;
            PL1ComboBox.Items.Clear();
            PL1ComboBox.Text = "";

            if (iPlatformIndex == 0)
            {
                iMaximumIndex = PackagePowerLimitLevel1.Length;
                for (int i = 0; i < iMaximumIndex; i++)
                    PL1ComboBox.Items.Add("Level" + i.ToString() + ", " + PackagePowerLimitLevel1[i] + " watts");
            }
            else if (iPlatformIndex == 1)
            {
                iMaximumIndex = PackagePowerLimitLevel2.Length;
                for (int i = 0; i < iMaximumIndex; i++)
                    PL1ComboBox.Items.Add("Level" + i.ToString() + ", " + PackagePowerLimitLevel2[i] + " watts");
            }
            else if (iPlatformIndex == 2)
            {
                iMaximumIndex = PackagePowerLimitLevel3.Length;
                for (int i = 0; i < iMaximumIndex; i++)
                    PL1ComboBox.Items.Add("Level" + i.ToString() + ", " + PackagePowerLimitLevel3[i] + " watts");
            }
            else if (iPlatformIndex == 3)
            {
                iMaximumIndex = PackagePowerLimitLevel4.Length;
                for (int i = 0; i < iMaximumIndex; i++)
                    PL1ComboBox.Items.Add("Level" + i.ToString() + ", " + PackagePowerLimitLevel4[i] + " watts");
            }

            CPUPL1Label.Text = "Current CPU PL1 : 0";

        }
    }
}
